package by.zerg.wtp.domain;

import java.io.IOException;
import java.io.StringWriter;
import java.net.SocketTimeoutException;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;
import java.util.zip.GZIPInputStream;

import org.apache.commons.io.IOUtils;
import org.apache.http.Header;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicHeader;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.params.CoreConnectionPNames;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;

import by.zerg.wtp.exception.BaseWeaktigthPosterException;
import by.zerg.wtp.exception.ConnectionFailedException;
import by.zerg.wtp.exception.ConnectionTimedOutException;

public class WeaktightPoster {
	public static final int PUBLIC = 0;
	public static final int FRIENDS_ONLY = 1;
	HttpClient client;
	Config config;

	static WeaktightPoster instance;

	private WeaktightPoster() throws BaseWeaktigthPosterException {
		this.config = Config.getInstance();
		client = new DefaultHttpClient();
		client.getParams().setParameter(
				CoreConnectionPNames.CONNECTION_TIMEOUT, 15000);
		client.getParams().setParameter(CoreConnectionPNames.SO_TIMEOUT, 15000);
		List<NameValuePair> loginForm = new ArrayList<NameValuePair>();
		loginForm.add(new BasicNameValuePair("username", config.getUsername()));
		loginForm.add(new BasicNameValuePair("password", config.getPassword()));
		loginForm.add(new BasicNameValuePair("redirect", config.getRedirect()));
		loginForm.add(new BasicNameValuePair("login", config.getLogin()));
		try {
			HttpResponse response = doPost(config.getLoginUrl(), loginForm,
					null);
			if (response.getFirstHeader("location").getValue().toLowerCase()
					.equals("http://weaktight.com/errorlogin")) {
				throw new ConnectionFailedException("Connection failed...");
			}
		} catch (SocketTimeoutException e) {
			throw new ConnectionTimedOutException("Connection timed out...");
		} catch (Throwable t) {
			throw new ConnectionFailedException("Connection failed...");
		}
	}

	public static void load() throws BaseWeaktigthPosterException {
		instance = new WeaktightPoster();
	}

	public static WeaktightPoster getInstance() {
		return instance;
	}

	private HttpResponse doPost(String url, List form, Header[] headers)
			throws ClientProtocolException, IOException {
		UrlEncodedFormEntity entity = new UrlEncodedFormEntity(form, "UTF-8");
		HttpPost httppost = new HttpPost(url);
		httppost.setHeaders(headers);
		httppost.setEntity(entity);
		HttpResponse response = client.execute(httppost);
		httppost.releaseConnection();
		return response;
	}

	private HttpGet doGet(String url, Header[] headers)
			throws ClientProtocolException, IOException {
		HttpGet httpget = new HttpGet(url);
		httpget.setHeaders(headers);
		return httpget;
	}

	public boolean tryHand(Hand hand) throws ConnectionTimedOutException {
		try {
			List<NameValuePair> postForm = new ArrayList<NameValuePair>();
			postForm.add(new BasicNameValuePair("hh", hand.getHistory()));
			Header[] headers = new Header[] {
					new BasicHeader("Accept-Encoding", "gzip, deflate"),
					new BasicHeader("X-Requested-With", "XMLHttpRequest") };
			HttpResponse response = doPost(config.getPostUrl(), postForm,
					headers);
			StringWriter writer = new StringWriter();
			IOUtils.copy(
					new GZIPInputStream(response.getEntity().getContent()),
					writer);
			StringTokenizer st = new StringTokenizer(writer.toString(), "{:,}");
			while (st.hasMoreTokens()) {
				String nextToken = st.nextToken().trim();
				if (nextToken.equals("\"error\"")
						&& !st.nextToken().trim().equals("null")) {
					return false;
				} else if (nextToken.equals("\"site\"")) {
					String site = st.nextToken().trim();
					hand.setSite(site.substring(1, site.length() - 1));
				} else if (nextToken.equals("\"game\"")) {
					hand.setLimit(st.nextToken().trim());
				}
			}
			System.out.println("site: " + hand.getSite());
			System.out.println("limit: " + hand.getLimit());
			return true;
		} catch (SocketTimeoutException e) {
			throw new ConnectionTimedOutException("Connection timed out...");
		} catch (Throwable t) {
			t.printStackTrace();
			return false;
		}
	}

	private void showRequest(HttpPost httppost) {
		System.out.println(httppost.getRequestLine());
		for (Header h : httppost.getAllHeaders()) {
			System.out.println(h);
		}
	}

	public String convertToHtml(Hand hand) {
		try {
			Header[] headers = new Header[] {
					new BasicHeader("Accept-Encoding", "gzip, deflate"),
					new BasicHeader("X-Requested-With", "XMLHttpRequest") };
			URIBuilder builder = new URIBuilder(config.getConvertUrl())
					// .addParameter("actioncombo", "0")
					.addParameter("convert", "1")
					// .addParameter("dbname", "")
					.addParameter("fmt", "12")
					.addParameter(
							"hand_id",
							hand.getLink().substring(
									hand.getLink().lastIndexOf("/") + 1))
					// .addParameter("o_chat", "on")
					// .addParameter("o_clearcache", "0")
					// .addParameter("o_compfolds", "on")
					.addParameter("o_markrel", "on")
					// .addParameter("o_marksd", "on")
					// .addParameter("o_profile_session_only", "on")
					.addParameter("o_results", "on")
					// .addParameter("o_txthname", config.getUsername())
					.addParameter("password", config.getPassword())
					// .addParameter("port", "").addParameter("readtext", "")
					.addParameter("username", config.getUsername());
			HttpGet httpget = doGet(builder.toString(), headers);
			HttpResponse response = client.execute(httpget);
			StringWriter writer = new StringWriter();
			IOUtils.copy(
					new GZIPInputStream(response.getEntity().getContent()),
					writer);
			httpget.releaseConnection();
			Document htmlDoc = Jsoup.parse(writer.toString());
			return "<html>" + htmlDoc.select("textarea").first().text()
					+ "</html>";
			// return htmlDoc.select("textarea").first().text();
		} catch (Throwable t) {
			return null;
		}
	}

	public void postHand(Hand hand, int visibility)
			throws ConnectionTimedOutException {
		try {
			List<NameValuePair> postForm = new ArrayList<NameValuePair>();
			postForm.add(new BasicNameValuePair("handhistory", hand
					.getHistory()));
			postForm.add(new BasicNameValuePair("handpot", config.getHandpot()));
			switch (visibility) {
			case PUBLIC:
				postForm.add(new BasicNameValuePair("o_pv_pub", "on"));
				break;
			default:
				postForm.add(new BasicNameValuePair("o_pvfr", "on"));
			}
			postForm.add(new BasicNameValuePair("df83", config.getDf83()));
			HttpResponse response = doPost(config.getPostUrl(), postForm, null);
			Header locationHeader = response.getFirstHeader("location");
			hand.setLink(locationHeader.getValue());
			Header[] headers = new Header[] { new BasicHeader(
					"Accept-Encoding", "gzip, deflate") };
			HttpGet httpget = doGet(hand.getLink(), headers);
			response = client.execute(httpget);
			StringWriter writer = new StringWriter();
			IOUtils.copy(
					new GZIPInputStream(response.getEntity().getContent()),
					writer);
			httpget.releaseConnection();

			Document htmlDoc = Jsoup.parse(writer.toString());
			String cards = "";
			try {
				Element htmlElement = htmlDoc.select("div.hd2").first()
						.select("li").last();

				for (Element cardElement : htmlElement.children()) {
					String card = cardElement.className();
					cards += card.substring(card.length() - 2) + " ";
				}
			} catch (Throwable t) {
				cards = "";
			}
			hand.setCards(cards);
			hand.setPot(htmlDoc.select("div.h1h").select("span.blue").text());
			System.out.println(hand.getPot());
			System.out.println(hand.cards);
		} catch (SocketTimeoutException e) {
			throw new ConnectionTimedOutException("Connection timed out...");
		} catch (Throwable t) {
			hand.setLink("ERROR");
			t.printStackTrace();
		}
	}
}
