package by.zerg.wtp.gui;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Desktop;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.net.URI;
import java.util.StringTokenizer;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingWorker;

import by.zerg.wtp.domain.Hand;
import by.zerg.wtp.domain.WeaktightPoster;
import by.zerg.wtp.exception.ConnectionTimedOutException;

public class HandPanel extends JPanel {
	Hand hand;
	WeaktightPoster poster;
	JLabel label;
	JTextField field;
	JCheckBox selectedCheckbox;
	JCheckBox publicCheckbox;
	JButton postButton;
	JButton previewButton;
	JButton openButton;
	JButton copyButton;
	JLabel loadingLabel;
	JPanel handPanel;

	public HandPanel(final MainFrame mainFrame, final Hand hand) {
		this.hand = hand;
		this.poster = WeaktightPoster.getInstance();

		MouseListener mouseListener = new MouseAdapter() {

			@Override
			public void mouseEntered(MouseEvent e) {
				activate();
			}

			@Override
			public void mouseExited(MouseEvent e) {
				deactivate();
			}

		};
		addMouseListener(mouseListener);

		label = new JLabel();
		label.addMouseListener(mouseListener);
		field = new JTextField();
		field.setEditable(false);
		field.addFocusListener(new FocusAdapter() {

			@Override
			public void focusGained(FocusEvent e) {
				field.selectAll();
			}

		});
		field.addMouseListener(new MouseAdapter() {

			@Override
			public void mousePressed(MouseEvent e) {
				field.selectAll();
			}

		});
		field.addMouseListener(mouseListener);
		selectedCheckbox = new JCheckBox();
		selectedCheckbox.addItemListener(new ItemListener() {

			@Override
			public void itemStateChanged(ItemEvent e) {
				mainFrame.handSelectionChanged();
			}

		});
		selectedCheckbox.addMouseListener(mouseListener);
		selectedCheckbox.setSelected(true);

		publicCheckbox = new JCheckBox();
		publicCheckbox.addMouseListener(mouseListener);

		postButton = new JButton(new ImageIcon("images" + File.separator
				+ "post.png"));
		postButton.setBorder(BorderFactory.createEmptyBorder());
		postButton.setContentAreaFilled(false);
		postButton.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
		postButton.setToolTipText("post");
		postButton.addMouseListener(mouseListener);
		postButton.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				mainFrame.count = 0;
				post(mainFrame);
			}

		});

		previewButton = new JButton(new ImageIcon("images" + File.separator
				+ "preview.png"));
		previewButton.setBorder(BorderFactory.createEmptyBorder());
		previewButton.setContentAreaFilled(false);
		previewButton.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
		previewButton.setToolTipText("preview");
		previewButton.addMouseListener(mouseListener);
		previewButton.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				Messenger.showPreviewDialog(mainFrame, hand);
			}

		});

		openButton = new JButton(new ImageIcon("images" + File.separator
				+ "open.png"));
		openButton.setBorder(BorderFactory.createEmptyBorder());
		openButton.setContentAreaFilled(false);
		openButton.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
		openButton.setToolTipText("open");
		openButton.addMouseListener(mouseListener);
		openButton.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				try {
					Desktop.getDesktop().browse(new URI(field.getText()));
				} catch (Throwable t) {
					t.printStackTrace();
				}
			}

		});

		copyButton = new JButton(new ImageIcon("images" + File.separator
				+ "copy.png"));
		copyButton.setBorder(BorderFactory.createEmptyBorder());
		copyButton.setContentAreaFilled(false);
		copyButton.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
		copyButton.setToolTipText("copy");
		copyButton.addMouseListener(mouseListener);
		copyButton.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				Clipboard clipboard = Toolkit.getDefaultToolkit()
						.getSystemClipboard();
				clipboard.setContents(new StringSelection(field.getText()),
						null);
			}

		});

		handPanel = new JPanel() {
			public static final int ACE = 0;
			public static final int KING = 1;
			public static final int QUEEN = 2;
			public static final int JACK = 3;
			public static final int TEN = 4;
			public static final int NINE = 5;
			public static final int EIGHT = 6;
			public static final int SEVEN = 7;
			public static final int SIX = 8;
			public static final int FIVE = 9;
			public static final int FOUR = 10;
			public static final int THREE = 11;
			public static final int TWO = 12;
			public static final int UNKNOWN = 13;

			public static final int SPADES = 0;
			public static final int HEARTS = 1;
			public static final int DIAMONDS = 2;
			public static final int CLUBS = 3;

			@Override
			public void paintComponent(Graphics g) {
				super.paintComponent(g);
				try {
					// setPreferredSize(new Dimension(0, 0));
					int i = 0;
					StringTokenizer st = new StringTokenizer(hand.getCards());
					while (st.hasMoreTokens()) {
						drawCard(g, st.nextToken(), i);
						i++;
					}
					setPreferredSize(new Dimension(20 * i, 28));
				} catch (Throwable t) {
					g.clearRect(0, 0, getWidth(), getHeight());
				}
			}

			public void drawCard(Graphics g, String card, int position) {
				String cardName = card.substring(0, 1);
				String suitName = card.substring(1, 2);
				Image deck = new ImageIcon("images" + File.separator
						+ "cards.png").getImage();
				int width = deck.getWidth(null);
				int height = deck.getHeight(null);
				int w = width / 14;
				int h = height / 4;
				int x = w * getCardNumber(cardName);
				int y = h * getCardSuit(suitName);
				g.drawImage(deck, position * 20, 0, position * 20 + 20,
						getHeight(), x, y, x + w, y + h, null);
			}

			public int getCardNumber(String card) {
				if (card.equals("A")) {
					return ACE;
				} else if (card.equals("K")) {
					return KING;
				} else if (card.equals("Q")) {
					return QUEEN;
				} else if (card.equals("J")) {
					return JACK;
				} else if (card.equals("T")) {
					return TEN;
				} else if (card.equals("9")) {
					return NINE;
				} else if (card.equals("8")) {
					return EIGHT;
				} else if (card.equals("7")) {
					return SEVEN;
				} else if (card.equals("6")) {
					return SIX;
				} else if (card.equals("5")) {
					return FIVE;
				} else if (card.equals("4")) {
					return FOUR;
				} else if (card.equals("3")) {
					return THREE;
				} else if (card.equals("2")) {
					return TWO;
				} else {
					return UNKNOWN;
				}
			}

			public int getCardSuit(String suit) {
				if (suit.equals("s")) {
					return SPADES;
				} else if (suit.equals("h")) {
					return HEARTS;
				} else if (suit.equals("d")) {
					return DIAMONDS;
				} else if (suit.equals("c")) {
					return CLUBS;
				} else {
					return UNKNOWN;
				}
			}

		};
		handPanel.addMouseListener(mouseListener);
		handPanel.setToolTipText(hand.getCards());

		JLabel siteLabel = new JLabel(new ImageIcon("images" + File.separator
				+ hand.getSite() + ".png"));
		// siteLabel.setPreferredSize(new Dimension(16, 16));
		siteLabel.addMouseListener(mouseListener);
		siteLabel.setToolTipText(hand.getSite());

		setLayout(new GridBagLayout());
		GridBagConstraints defaultConstraints = new GridBagConstraints();
		defaultConstraints.insets = new Insets(5, 5, 5, 5);
		GridBagConstraints fieldConstraints = new GridBagConstraints();
		fieldConstraints.fill = GridBagConstraints.HORIZONTAL;
		fieldConstraints.anchor = GridBagConstraints.WEST;
		fieldConstraints.weightx = 1;
		fieldConstraints.insets = new Insets(5, 5, 5, 5);

		loadingLabel = new JLabel();
		ImageIcon icon = new ImageIcon("images" + File.separator
				+ "loading.gif");
		loadingLabel.setIcon(icon);
		loadingLabel.setVisible(false);

		add(loadingLabel, defaultConstraints);
		add(siteLabel, defaultConstraints);
		add(selectedCheckbox, defaultConstraints);
		add(publicCheckbox, defaultConstraints);
		add(handPanel, defaultConstraints);
		add(label, defaultConstraints);
		add(field, fieldConstraints);
		add(postButton, defaultConstraints);
		add(copyButton, defaultConstraints);
		add(previewButton, defaultConstraints);
		add(openButton, defaultConstraints);
	}

	public void loading() {
		loadingLabel.setVisible(true);
		field.setEnabled(false);
		postButton.setEnabled(false);
		previewButton.setEnabled(false);
		openButton.setEnabled(false);
		copyButton.setEnabled(false);
		selectedCheckbox.setEnabled(false);
		publicCheckbox.setEnabled(false);
	}

	public void activate() {
		setBackground(Color.YELLOW);
	}

	public void deactivate() {
		setBackground(null);
	}

	public void setLink(String link) {
		loadingLabel.setVisible(false);
		field.setText(link);
		field.setEnabled(true);
		postButton.setEnabled(true);
		previewButton.setEnabled(true);
		openButton.setEnabled(true);
		copyButton.setEnabled(true);
		selectedCheckbox.setEnabled(true);
		publicCheckbox.setEnabled(true);
	}

	public boolean isSelected() {
		return selectedCheckbox.isSelected();
	}

	public void post(final MainFrame mainFrame) {
		if (!isVisible()) {
			return;
		}
		new SwingWorker() {

			@Override
			protected Object doInBackground() {
				loading();
				synchronized (poster) {
					try {
						poster.postHand(
								hand,
								publicCheckbox.isSelected() ? WeaktightPoster.PUBLIC
										: WeaktightPoster.FRIENDS_ONLY);
					} catch (ConnectionTimedOutException e) {
						switch (Messenger
								.showConnectionTimedOutDialog(mainFrame)) {
						case Messenger.TRY_AGAIN:
							return doInBackground();
						}
					}
				}
				setLink(hand.getLink());
				label.setText("( " + hand.getLimit() + " )" + "  [ "
						+ hand.getPot() + " ]");
				handPanel.repaint();
				return null;
			}

			@Override
			protected void done() {
				mainFrame.postPerformed();
			}

		}.execute();
	}
}
