package by.zerg.wtp.gui;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.filechooser.FileFilter;

import by.zerg.wtp.domain.Config;
import by.zerg.wtp.domain.Hand;
import by.zerg.wtp.domain.WeaktightPoster;
import by.zerg.wtp.exception.ConfigNotFoundException;

public class MainFrame extends JFrame {
	int count;
	WeaktightPoster poster;
	List<HandPanel> handsPanels;
	List<Hand> hands;
	JLabel status;
	JButton loadButton;
	JButton postAllButton;
	JPanel handsPanel;
	FiltersPanel filtersPanel;

	public MainFrame() {
		hands = new ArrayList<Hand>();
		handsPanels = new ArrayList<HandPanel>();
	}

	private List<Hand> parseHands(File file) throws IOException {
		List<Hand> result = new ArrayList<Hand>();
		BufferedReader br = new BufferedReader(new InputStreamReader(
				new FileInputStream(file)));
		String line = null;
		String history = null;
		while ((line = br.readLine()) != null) {
			if (line.contains("Hand #")) {
				if (history != null) {
					result.add(new Hand(history));
				}
				history = new String();
			}
			if (history != null) {
				history += line + "\r\n";
			}
		}
		if (history != null) {
			result.add(new Hand(history));
		}
		br.close();
		return result;
	}

	private void parseHands(File[] files) {
		for (File file : files) {
			if (file.isDirectory()) {
				parseHands(file.listFiles());
			} else if (file.getName().toLowerCase().endsWith(".txt")) {
				try {
					hands.addAll(parseHands(file));
				} catch (Throwable t) {
					// TODO Auto-generated catch block
					t.printStackTrace();
				}
			}
		}
	}

	// private String fileToString(File file) throws IOException {
	// String result = new String();
	// BufferedReader br = new BufferedReader(new InputStreamReader(
	// new FileInputStream(file)));
	// String line = null;
	// while ((line = br.readLine()) != null) {
	// result += line + "\r\n";
	// }
	// br.close();
	// return result;
	// }

	public void addHandPanel(HandPanel p) {
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 0;
		c.gridy = handsPanels.size();
		c.weightx = 1;
		handsPanel.add(p, c);
	}

	private void load(File[] inputs) {
		handsPanel.removeAll();
		// File[] files = inputDir.listFiles();
		// int size = files.length;

		hands = new ArrayList<Hand>();
		handsPanels = new ArrayList<HandPanel>();
		parseHands(inputs);
		int size = hands.size();
		if (size > 0) {
			for (int i = 0; i < size; i++) {
				try {
					// String history = fileToString(files[i]);
					// hands[i] = new Hand(history);
					HandPanel newHandPanel = new HandPanel(this, hands.get(i));
					handsPanels.add(newHandPanel);
					// GridBagConstraints c = new GridBagConstraints();
					// c.fill = GridBagConstraints.HORIZONTAL;
					// c.gridx = 0;
					// c.gridy = i;
					// c.weightx = 1;
					// handsPanel.add(handsPanels.get(i), c);
					addHandPanel(newHandPanel);
				} catch (Throwable t) {
					t.printStackTrace();
				}
			}
			postAllButton.setEnabled(true);
			// filtersPanel.setVisible(true);
		} else {
			postAllButton.setEnabled(false);
			// filtersPanel.setVisible(false);
		}
		status.setText(size + " hands found...");
		filtersPanel.selectAllCheckbox.setSelected(true);
		// repaint();
		// pack();
	}

	private void postSelected() {
		// new SwingWorker() {
		//
		// @Override
		// protected Object doInBackground() throws Exception {
		// int count = 0;
		for (int i = 0; i < hands.size(); i++) {
			HandPanel p = handsPanels.get(i);
			if (p.isVisible() && p.isSelected()) {
				count++;
				p.loading();
			}
		}
		if (count != 0) {
			status.setText("0/" + hands.size() + " hands posted...");
			count = 0;
			for (int i = 0; i < hands.size(); i++) {
				HandPanel p = handsPanels.get(i);
				if (p.isSelected()) {
					// count++;
					// p.loading();
					// poster.postHand(hands.get(i));
					// p.setLink(hands.get(i).getLink());
					p.post(this);
					// status.setText(count + "/" + hands.size()
					// + " hands posted...");
				}
			}
			// pack();
		}
		// return null;
		// }
		//
		// @Override
		// protected void done() {
		//
		// }
		//
		// }.execute();
	}

	public void postPerformed() {
		status.setText(++count + "/" + hands.size() + " hands posted...");
	}

	public void init() {
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		setTitle("Weaktight Poster v1.3rc1 by Hitman");

		final JFileChooser fc = new JFileChooser();
		fc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
		fc.setMultiSelectionEnabled(true);
		fc.setFileFilter(new FileFilter() {

			@Override
			public String getDescription() {
				return "HandHistory";
			}

			@Override
			public boolean accept(File f) {
				return f.isDirectory()
						|| f.getName().toLowerCase().endsWith(".txt");
			}
		});

		filtersPanel = new FiltersPanel(this);
		// filtersPanel.setVisible(false);

		handsPanel = new JPanel(new GridBagLayout());
		JPanel centralPanel = new JPanel(new BorderLayout());
		centralPanel.add(handsPanel, BorderLayout.NORTH);
		JScrollPane scroller = new JScrollPane(centralPanel);
		scroller.getVerticalScrollBar().setUnitIncrement(30);
		// add(scroller);

		loadButton = new JButton("Load");
		loadButton.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				int returnVal = fc.showOpenDialog(null);
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					File[] inputs = fc.getSelectedFiles();
					load(inputs);
				}
			}

		});

		postAllButton = new JButton("Post Selected");
		postAllButton.setEnabled(false);
		postAllButton.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				postSelected();
			}

		});

		JPanel postPanel1 = new JPanel();
		postPanel1.add(loadButton);
		postPanel1.add(postAllButton);
		JPanel postPanel2 = new PostPanel(this);
		// add(topPanel, BorderLayout.NORTH);

		status = new JLabel(" ");
		// add(status, BorderLayout.SOUTH);

		// pack();

		setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		// c.gridx = 0;
		c.gridy = 0;
		c.weightx = 1;
		// c.anchor = GridBagConstraints.NORTH;
		add(postPanel1, c);

		c.gridy = 1;
		add(new JSeparator(JSeparator.HORIZONTAL), c);

		c.gridy = 2;
		c.weighty = 0;
		add(postPanel2, c);

		c.gridy = 3;
		add(new JSeparator(JSeparator.HORIZONTAL), c);

		c.gridy = 4;
		// c.anchor = GridBagConstraints.CENTER;
		add(filtersPanel, c);

		c.gridy = 6;
		// c.anchor = GridBagConstraints.SOUTH;
		add(status, c);

		c.gridy = 5;
		c.fill = GridBagConstraints.BOTH;
		c.weighty = 1;
		// c.anchor = GridBagConstraints.CENTER;
		add(scroller, c);

		setSize(800, 600);
		setMinimumSize(getSize());
		setLocationRelativeTo(null);
		setVisible(true);
		postPanel2.requestFocus();
	}

	public void handSelectionChanged() {
		boolean selected = false;
		for (HandPanel hp : handsPanels) {
			if (hp.selectedCheckbox.isSelected()) {
				selected = true;
				break;
			}
		}
		postAllButton.setEnabled(selected);
	}

	public static void main(String[] args) {
		try {
			javax.swing.UIManager.LookAndFeelInfo[] installedLookAndFeels = javax.swing.UIManager
					.getInstalledLookAndFeels();
			for (int idx = 0; idx < installedLookAndFeels.length; idx++) {
				if ("Nimbus".equals(installedLookAndFeels[idx].getName())) {
					javax.swing.UIManager
							.setLookAndFeel(installedLookAndFeels[idx]
									.getClassName());
					break;
				}
			}
		} catch (Throwable t) {
		}

		final MainFrame mainFrame = new MainFrame();
		mainFrame.init();

		JDialog dialog = Messenger.showLoadConfigDialog(mainFrame);
		try {
			Config.load();
			dialog.dispose();
			mainFrame.getGlassPane().setVisible(false);
		} catch (ConfigNotFoundException e) {
			dialog.dispose();
			Messenger.showLoadConfigErrorDialog(mainFrame);
			System.exit(1);
		}

		while (true) {
			try {
				dialog = Messenger.showConnectionDialog(mainFrame);
				WeaktightPoster.load();
				dialog.dispose();
				break;
			} catch (Throwable t) {
				dialog.dispose();
				switch (Messenger.showConnectionFailedDialog(mainFrame)) {
				case Messenger.TRY_AGAIN:
					continue;
				default:
					System.exit(1);
				}
			}
		}
	}
}
