package by.zerg.wtp.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.io.File;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.KeyStroke;

import by.zerg.wtp.domain.Hand;
import by.zerg.wtp.domain.WeaktightPoster;

class StatusDialog extends JDialog {

	public StatusDialog(JFrame parent, String message) {
		super(parent, true);
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		setUndecorated(true);
		JPanel panel = new JPanel(new GridBagLayout());
		panel.setBorder(BorderFactory.createLineBorder(Color.GRAY, 3));
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.insets = new Insets(30, 30, 30, 30);
		panel.add(new JLabel(message), gbc);
		add(panel);
		pack();
		setLocationRelativeTo(parent);
	}

}

class PreviewDialog extends JDialog {
	Hand hand;
	JLabel previewLabel;
	JLabel loadingLabel;
	WeaktightPoster poster;

	public PreviewDialog(JFrame parent, Hand hand) {
		super(parent, "Hand Preview", true);
		poster = WeaktightPoster.getInstance();
		this.hand = hand;
		loadingLabel = new JLabel(new ImageIcon("images" + File.separator
				+ "loading.gif"));
		previewLabel = new JLabel();
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		add(loadingLabel);
		pack();
		setLocationRelativeTo(parent);
		setResizable(false);
		getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
				KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), "CANCEL"); //$NON-NLS-1$
		getRootPane().getActionMap().put("CANCEL", new AbstractAction() { //$NON-NLS-1$

					public void actionPerformed(ActionEvent e) {
						dispose();
					}

				});
	}

	@Override
	public void setVisible(final boolean b) {
		new Thread() {
			@Override
			public void run() {
				PreviewDialog.super.setVisible(b);
			}
		}.start();
		if (b) {
			synchronized (poster) {
				previewLabel.setText(poster.convertToHtml(hand));
				// add(new JEditorPane("text/html",
				// poster.convertToHtml(hand)));
			}
			add(new JScrollPane(previewLabel));
			// add(previewLabel);
			loadingLabel.setVisible(false);
			previewLabel.setVisible(true);
			pack();
			setLocationRelativeTo(getParent());
			setResizable(true);
		}
	}
}

public class Messenger {
	public static final int ABORT = JOptionPane.CLOSED_OPTION;
	public static final int TRY_AGAIN = JOptionPane.YES_OPTION;
	public static final int CANCEL = JOptionPane.NO_OPTION;
	public static final int EXIT = JOptionPane.NO_OPTION;
	private static final Object[] OPTIONS1 = { "Try Again", "Exit" };
	private static final Object[] OPTIONS2 = { "Try Again", "Cancel" };

	public static void showLoadConfigErrorDialog(Component parent) {
		JOptionPane.showMessageDialog(parent, "Can't open config", "Error",
				JOptionPane.ERROR_MESSAGE);
	}

	public static int showConnectionFailedDialog(Component parent) {
		return JOptionPane.showOptionDialog(parent,
				"Can't connect to weaktight.com", "Connection Failed",
				JOptionPane.YES_NO_OPTION, JOptionPane.ERROR_MESSAGE, null,
				OPTIONS1, OPTIONS1[0]);
	}

	public static int showConnectionTimedOutDialog(Component parent) {
		return JOptionPane.showOptionDialog(parent, "Connection Timed Out",
				"Connection Timed Out", JOptionPane.YES_NO_OPTION,
				JOptionPane.ERROR_MESSAGE, null, OPTIONS2, OPTIONS2[0]);
	}

	private static JDialog showMessageDialog(JFrame parent, String message) {
		final JDialog dialog = new StatusDialog(parent, message);
		new Thread() {

			@Override
			public void run() {
				dialog.setVisible(true);
			}

		}.start();
		return dialog;
	}

	public static JDialog showPreviewDialog(JFrame parent, Hand hand) {
		final JDialog dialog = new PreviewDialog(parent, hand);
		new Thread() {

			@Override
			public void run() {
				dialog.setVisible(true);
			}

		}.start();
		return dialog;
	}

	public static JDialog showLoadConfigDialog(JFrame parent) {
		return showMessageDialog(parent, "Loading config. Please wait...");
	}

	public static JDialog showConnectionDialog(JFrame parent) {
		return showMessageDialog(parent,
				"Connecting to weaktight.com. Please wait...");
	}

	public static JDialog showHandVerificationDialog(JFrame parent) {
		return showMessageDialog(parent, "Verifying Hand. Please wait...");
	}
}
