package by.zerg.wtp.gui;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Date;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingConstants;
import javax.swing.SwingWorker;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import by.zerg.wtp.domain.Hand;
import by.zerg.wtp.domain.WeaktightPoster;
import by.zerg.wtp.exception.ConnectionTimedOutException;

public class PostPanel extends JPanel {
	JCheckBox publicCheckbox;
	JButton postButton;
	JTextArea postArea;
	Date modifyDate;
	MainFrame mainFrame;
	Hand newHand;

	public PostPanel(final MainFrame mainFrame) {
		this.mainFrame = mainFrame;
		modifyDate = new Date();

		publicCheckbox = new JCheckBox("Public:");
		publicCheckbox.setHorizontalTextPosition(SwingConstants.LEFT);

		JButton clearButton = new JButton("Clear");
		clearButton.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				postArea.setText(null);
				postArea.requestFocus();
			}

		});

		postButton = new JButton("Post");
		postButton.setEnabled(false);
		postButton.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				try {
					mainFrame.hands.add(newHand);
					HandPanel newHandPanel = new HandPanel(mainFrame, newHand);
					mainFrame.handsPanels.add(newHandPanel);
					mainFrame.addHandPanel(newHandPanel);
					newHandPanel.publicCheckbox.setSelected(publicCheckbox
							.isSelected());
					newHandPanel.post(mainFrame);
					newHandPanel.selectedCheckbox.setSelected(false);
					postArea.setText(null);
					postArea.requestFocus();
					postButton.setEnabled(false);
				} catch (Throwable t) {
					t.printStackTrace();
				}
			}

		});

		postArea = new JTextArea(10, 20);
		postArea.getDocument().addDocumentListener(new DocumentListener() {

			@Override
			public void removeUpdate(DocumentEvent e) {
				tryHand();
			}

			@Override
			public void insertUpdate(DocumentEvent e) {
				tryHand();
			}

			@Override
			public void changedUpdate(DocumentEvent e) {
				tryHand();
			}

		});
		postArea.addFocusListener(new FocusAdapter() {

			@Override
			public void focusGained(FocusEvent e) {
				postArea.selectAll();
			}

		});
		postArea.addMouseListener(new MouseAdapter() {

			@Override
			public void mouseClicked(MouseEvent e) {
				postArea.selectAll();
			}

		});

		setLayout(new GridBagLayout());
		GridBagConstraints defaultConstraints = new GridBagConstraints();
		defaultConstraints.insets = new Insets(5, 5, 5, 5);
		GridBagConstraints fieldConstraints = new GridBagConstraints();
		fieldConstraints.fill = GridBagConstraints.BOTH;
		// fieldConstraints.anchor = GridBagConstraints.WEST;
		fieldConstraints.weightx = 1;
		fieldConstraints.weighty = 1;
		fieldConstraints.gridheight = 3;
		fieldConstraints.insets = new Insets(5, 5, 5, 5);
		// fieldConstraints.ipadx = 2;
		// fieldConstraints.ipady = 3;
		add(new JScrollPane(postArea), fieldConstraints);
		defaultConstraints.fill = GridBagConstraints.HORIZONTAL;
		defaultConstraints.gridx = 1;
		defaultConstraints.gridy = 0;
		defaultConstraints.anchor = GridBagConstraints.PAGE_END;
		add(publicCheckbox, defaultConstraints);
		defaultConstraints.gridx = 1;
		defaultConstraints.gridy = 1;
		add(clearButton, defaultConstraints);
		defaultConstraints.gridx = 1;
		defaultConstraints.gridy = 2;
		add(postButton, defaultConstraints);
	}

	@Override
	public void requestFocus() {
		postArea.requestFocus();
	}

	private void tryHand() {
		String history = postArea.getText();
		if (history == null || history.trim().length() == 0) {
			return;
		}
		new SwingWorker() {

			WeaktightPoster poster = WeaktightPoster.getInstance();
			JDialog dialog = Messenger.showHandVerificationDialog(mainFrame);

			@Override
			protected Object doInBackground() {
				newHand = new Hand(postArea.getText());
				synchronized (poster) {
					try {
						postButton.setEnabled(poster.tryHand(newHand));
					} catch (ConnectionTimedOutException e) {
						switch (Messenger
								.showConnectionTimedOutDialog(mainFrame)) {
						case Messenger.TRY_AGAIN:
							return doInBackground();
						}
					}
				}
				return null;
			}

			@Override
			protected void done() {
				dialog.dispose();
			}

		}.execute();
	}
	// private void tryHand() {
	// String history = postArea.getText();
	// if (history == null || history.trim().length() == 0) {
	// return;
	// }
	// // if (new Date().getTime() - modifyDate.getTime() > 1000) {
	// final JDialog dialog = Messenger.showHandVerificationDialog(mainFrame);
	// SwingUtilities.invokeLater(new Runnable() {
	//
	// @Override
	// public void run() {
	// newHand = new Hand(postArea.getText());
	// WeaktightPoster poster = WeaktightPoster.getInstance();
	// synchronized (poster) {
	// postButton.setEnabled(poster.tryHand(newHand));
	// }
	// dialog.dispose();
	// }
	//
	// });
	// // }
	// // modifyDate = new Date();
	// }
}
